/* CEC (HDMI Consumer Electronic Control) application 
 Copyright (c) 2009 Michael Dreher <michael(at)5dot1.de>  All right reserved.
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "local_defs.h"

void loop()
{
  //delay(10);
  processSerialInput();

  byte b = checkCaptureFlags();
  if(b == CEC_STATE_RECEIVED)
  {
    debug_P(PSTR("Message: "));
    for(int i = 0; i < CECmsgLen; i++)
    {
      Serial.print(CECmsg[i], HEX);
      Serial.print(' ');
    }
    Serial.println();
    startIrqCapture();
  }
  else if(b == CEC_STATE_ERROR)
  {
    debug_P(PSTR("Error "));
    Serial.print(CECErrNo, DEC);
    Serial.print(' ');
    Serial.print(CECErrInfo, DEC);
    Serial.print(' ');
    Serial.print(CECmsgLen, DEC);
    Serial.print(' ');
    Serial.print(CECBitCount, DEC);
    Serial.println();
    startIrqCapture();
  }
}

void waitUntilMicros(unsigned long *pT0, unsigned long len)
{
  unsigned long endTime = *pT0 + len;
  while(micros() < endTime)
  {
    ;
  }
  *pT0 = endTime;
}

byte CECprint(uint8_t *data, uint8_t len)
{
  unsigned long t0 = micros();

  // start bit
  sbi(CEC_OUT_PORT, CEC_OUT_BIT_NR); // output: LOW (active) state
  waitUntilMicros(&t0, 3700);
  cbi(CEC_OUT_PORT, CEC_OUT_BIT_NR); // output: HIGH (idle) state
  waitUntilMicros(&t0, 800);

  bool ack = true;
  int8_t l = len;
  //while(ack && l--)
  while(l--)
  {
    uint16_t v = *(data++) << 1 | (l == 0); // data + EOM bit
    for(int8_t i = 8; i >= 0; i--)
    {
      bool hi = v & _BV(i);

      sbi(CEC_OUT_PORT, CEC_OUT_BIT_NR); // output: LOW (active) state
      waitUntilMicros(&t0, hi ? 500 : 1500);

      cbi(CEC_OUT_PORT, CEC_OUT_BIT_NR); // output: HIGH (idle) state
      waitUntilMicros(&t0, hi ? 1800 : 900);
    }

    // ACK bit
    sbi(CEC_OUT_PORT, CEC_OUT_BIT_NR); // output: LOW (active) state
    waitUntilMicros(&t0, 500);

    cbi(CEC_OUT_PORT, CEC_OUT_BIT_NR); // output: HIGH (idle) state
    waitUntilMicros(&t0, 500);

    // read ack from follower
    ack = ack && ((CEC_IN_PIN >> CEC_IN_BIT_NR) & 0x01);

    waitUntilMicros(&t0, 1400);
  }

  if(ack)
    return 0;

  return 1;
}


void processSerialInput(void)
{
  if(Serial.available())
  {
    unsigned char ch = Serial.read();

    switch(ch)
    {
    case 'r':
      startIrqCapture();
      debugln_P(PSTR("Restarted CEC state machine"));
      break;

    case '1':
      cbi(CEC_OUT_PORT, CEC_OUT_BIT_NR);
      debugln_P(PSTR("HI"));
      break;

    case '2':
      sbi(CEC_OUT_PORT, CEC_OUT_BIT_NR);
      debugln_P(PSTR("LO"));
      break;

    case '+':
      if(CECOwnAddress < 14)
        CECOwnAddress++;
      debug_P(PSTR("New address:"));
      Serial.print(CECOwnAddress, DEC);
      Serial.println();
      break;

    case '-':
      if(CECOwnAddress > 0)
        CECOwnAddress--;
      debug_P(PSTR("New address:"));
      Serial.print(CECOwnAddress, DEC);
      Serial.println();
      break;

    case 'q': // send to ourselve
    case 'Q': // broadcast
      {
        uint8_t data[] = {
          0x00
        };
        data[0] = (CECOwnAddress << 4) | ((ch & 0x20) ? CECOwnAddress : 0x0F); 
        if(CECprint(data, sizeof(data)) == 0)
          debugln_P(PSTR("Send ok"));
        else
          debugln_P(PSTR("Send NAK"));
      }
      break;

    case 'w': // send to ourselve
    case 'W': // broadcast
      {
        uint8_t data[] = {
          0x00, 0x12, 0x34,
        };
        data[0] = (CECOwnAddress << 4) | ((ch & 0x20) ? CECOwnAddress : 0x0F); 
        if(CECprint(data, sizeof(data)) == 0)
          debugln_P(PSTR("Send ok"));
        else
          debugln_P(PSTR("Send NAK"));
      }
      break;

    case 'e': // send to ourselve
    case 'E': // broadcast
      {
        uint8_t data[] = {
          0x00, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCE, 0xEF, 0x00, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB
        };
        data[0] = (CECOwnAddress << 4) | ((ch & 0x20) ? CECOwnAddress : 0x0F); 
        if(CECprint(data, sizeof(data)) == 0)
          debugln_P(PSTR("Send ok"));
        else
          debugln_P(PSTR("Send NAK"));
      }
      break;

      // Standby to broadcast address
    case 'a':
      {
        uint8_t data[] = {
          0x00, 0x36
        };
        data[0] = (CECOwnAddress << 4) | 0x0F; // send to broadcast address
        if(CECprint(data, sizeof(data)) == 0)
          debugln_P(PSTR("Send broadcast Standby ok"));
        else
          debugln_P(PSTR("Send broadcast Standby NAK"));
      }
      break;

      // Standby to TV address
    case 's':
      {
        uint8_t data[] = {
          0x00, 0x36
        };
        data[0] = (CECOwnAddress << 4) | 0x00; // send to TV address
        if(CECprint(data, sizeof(data)) == 0)
          debugln_P(PSTR("Send TV Standby ok"));
        else
          debugln_P(PSTR("Send TV Standby NAK"));
      }
      break;

      // Image View On to TV address
    case 'd':
      {
        uint8_t data[] = {
          0x00, 0x04
        };
        data[0] = (CECOwnAddress << 4) | 0x00; // send to TV address
        if(CECprint(data, sizeof(data)) == 0)
          debugln_P(PSTR("Send 'Image View On' ok"));
        else
          debugln_P(PSTR("Send 'Image View On' NAK"));
      }
      break;

    }
  }
}


void setup()
{
  CEC_init();	// CEC specific initialization
  Serial.begin(115200);                 // connect to the serial port

  // application specific initialization
  sbi(LEDDDR, LED);

  for(unsigned char i = 0; i < 5*2; i++)
  {
    toggleLED();
    delay(100);
  }

}




























